from pydantic import BaseModel, Field

from xmpp_api.db.bot import JIDType


class BotInformation(BaseModel):
    """
    Information about a bot.
    """

    # The bot's ID
    id: str

    # The bot's name
    name: str

    # The bot's localpart
    localpart: str

    # The bot's description
    description: str | None

    # List of constraints on the bot
    constraints: list["BotConstraint"]


class AllowedJidInformation(BaseModel):
    """
    Information about an allowed JID
    """

    # The JID that a message would be sent to.
    jid: str

    # The token that the bot must use in order to send
    # messages to this JID.
    token: str


class BotConstraint(BaseModel):
    """
    Base-class for bot constraints.
    """


class BotDomainConstraint(BotConstraint):
    """
    Bot constraint that prevents the bot from sending messages to
    domains that are not listed in @domains.
    """

    domains: list[str]


class CreateBotRequest(BaseModel):
    """
    Model for a bot creation request.
    """

    # The bot's name
    name: str

    # The bot's description
    description: str | None = Field(default=None)

    # List of constraints
    constraints: list[BotConstraint] = Field(default_factory=list)

    # The localpart of the bot
    localpart: str


class BotCreationResponse(BotInformation):
    """
    The response to a successful bot creation request.
    """

    # The bot's token
    token: str


class AddJidRequest(BaseModel):
    """
    The request to add a JID to a bot.
    """

    # The JID that the message will be sent to
    jid: str

    # The JID type. If set, then we will not discover it automatically
    type: JIDType | None = Field(default=None)


class AddJidResponse(BaseModel):
    """
    The response to a successful JID addition.
    """

    # The token that identifies the previously specified JID.
    token: str


class SendMessageRequest(BaseModel):
    """
    Model for the request to send a message to a JID.
    """

    # The message content
    body: str


class GetBotInformation(BotInformation):
    """
    Response for an information request about a given bot.
    """

    jids: list[AllowedJidInformation]
