from typing import Annotated, Literal

from fastapi import Depends
from slixmpp.componentxmpp import ComponentXMPP
from slixmpp.jid import JID
from slixmpp.exceptions import IqError, IqTimeout

from xmpp_api.config.config import ConfigDep


class XmppApiComponent(ComponentXMPP):
    """
    The XMPP server component that sends the messages
    """

    # The component's bare JID
    _jid: str

    # Singleton instance
    _instance = None

    def __init__(self, jid: str, secret: str, host: str, port: int):
        super().__init__(jid, secret, host, port)
        self._jid = jid

        # Register plugins
        self.register_plugin("xep_0030")

        # Event handlers
        self.add_event_handler("disconnected", self.on_disconnected)
        self.add_event_handler("connected", self.on_connected)

    @staticmethod
    def of(config: ConfigDep) -> "XmppApiComponent":
        if XmppApiComponent._instance is None:
            host, port = config.component.server.split(":")
            XmppApiComponent._instance = XmppApiComponent(
                config.component.jid,
                config.component.secret,
                host,
                int(port),
            )
        return XmppApiComponent._instance

    def on_disconnected(self, event):
        # Reconnect
        self.connect()

    def on_connected(self, event):
        # TODO: Join all groupchats that we know of
        pass

    def run(self):
        # NOTE: We do not have to deal with asyncio here because we get that
        #       due to fastapi for free!
        self.connect()

    async def get_entity_type(
        self, jid: JID
    ) -> Literal["groupchat"] | Literal["account"] | None:
        try:
            info = await self.plugin["xep_0030"].get_info(jid=jid)
            if "http://jabber.org/protocol/muc" in info["disco_info"]["features"]:
                return "groupchat"
            return "account"
        except IqError:
            return None
        except IqTimeout:
            return None

    def send_direct_message(self, localpart: str, nick: str, recipient: str, body: str):
        self.send_message(
            mto=JID(recipient),
            mfrom=JID(f"{localpart}@{self._jid}"),
            mtype="chat",
            mbody=body,
            mnick=nick,
        )


def get_xmpp_component(config: ConfigDep) -> XmppApiComponent:
    return XmppApiComponent.of(config)


XmppApiComponentDep = Annotated[XmppApiComponent, Depends(get_xmpp_component)]
